// Implementation of the Network Animation Viewer Model
// George F. Riley, Georgia Tech, Spring 2009
// Modifications Lukasz Kiedrowski <kiedrowski@itti.com.pl>

#include <iostream>
#include "NetModel.h"

using namespace std;

// NetNode data/methods
NetNode::NetNode()
  : id(-1), x(0), y(0), z(0), color('b'), ip_id(0), label(false), shape(CIRCLE),
    visible(false), exists(false), lastPacketLbtxTime(0)
{ // Default constructor is only used when indexing the map of
  // all nodes with an invalid id; this gives a node with exists = false.
}

NetNode::NetNode(NodeId_t id0, Meters_t x0, Meters_t y0, Meters_t z0)
  : id(id0), x(x0), y(y0), z(z0), color('b'), ip_id(0), label(false), shape(CIRCLE),
    visible(true), exists(true), lastPacketLbtxTime(0)
{
}

NetNode::NetNode(NodeId_t id0, Meters_t x0, Meters_t y0, Meters_t z0, char color0)
  : id(id0), x(x0), y(y0), z(z0), color(color0), ip_id(0), label(false), shape(CIRCLE),
    visible(true), exists(true), lastPacketLbtxTime(0)
{
}

void NetNode::SetShape(NodeShape_t s)
{
  shape = s;
}


void NetNode::Show()
{
  visible = true;
}

void NetNode::Hide()
{
  visible = false;
}

NetWiredLink NetNode::FindLink(NodeId_t n2Id)
{
  if (exists)
    {
      for (Size_t i = 0; i < links.size(); ++i)
        {
          if (links[i].n2 == n2Id) return links[i];
        }
    }
  // Not found, return empty link
  return NetWiredLink();
}

void NetNode::AddLink(const NetWiredLink& lk)
{
  links.push_back(lk);
}

bool NetNode::OnOffNodeLabel ()
{
  label=!label;
  return label;
}

QPointF NetNode::GetLoc2d() const
{
  return QPointF(x, y);
}

NetWiredLink::NetWiredLink()
  : exists(false)
{ // Allow an erroneous return from GetLink
}

NetWiredLink::NetWiredLink(NodeId_t nd1, NodeId_t nd2)
  : n1(nd1), n2(nd2), exists(true)
{
}

// Wired Packet Members and Methods

NetWiredPacket::NetWiredPacket(NodeId_t n1, NodeId_t n2,
                                 Time_t fbTx0, Time_t lbTx0,
                                 Time_t fbRx0, Time_t lbRx0)
  : fbTx(fbTx0), lbTx(lbTx0), fbRx(fbRx0), lbRx(lbRx0),
    txNode(n1), rxNode(n2)
{
}

// NetModel methods

// Constructor
NetModel::NetModel() : currentTime(0), circle_radius(0), view(nil) 
{
  // Nothing else needed  
}

void NetModel::SetView(NetView* v)
{
  view = v;
}

QRectF NetModel::GetBoundingRect()
{
  QRectF br(topLeft, QSizeF());
  // Add 10% borders
  float w = bottomRight.x() - topLeft.x();
  float h = bottomRight.y() - topLeft.y();
  br.setWidth (w * 1.1);//org
  br.setHeight(h * 1.1);//org
  br.moveTopLeft(topLeft - QPointF(w * 0.05, h * 0.05)); //org
  //
  //br.setWidth (w);
  //br.setHeight(h);
  //br.moveTopLeft(topLeft - QPointF(circle_radius, circle_radius));
  return br;
}

// SetBoundingRect sets the minimum size of the network node placement.
// It will grow as needed as nodes are added
void NetModel::SetBoundingRect(QRectF br)
{
  topLeft = br.topLeft();
  bottomRight = br.bottomRight();
}

//zmiany tu
void NetModel::AddNode(NodeId_t id,
                             Meters_t x, Meters_t y, Meters_t z)
{
  //cout << "NN " << 0.0 << " " << x0 << " " << y0 << endl;
  if (topLeft == QPointF(0,0) && bottomRight == QPointF(0,0))
    { // First node, set ul and lr
      topLeft = QPointF(x, y);
      bottomRight = QPointF(x, y);
    }
  else
    {
      //org
      if (x < topLeft.x())  topLeft.setX(x);
      if (y < topLeft.y())  topLeft.setY(y);
      if (x > bottomRight.x()) bottomRight.setX(x);
      if (y > bottomRight.y()) bottomRight.setY(y);
    }
  //allNodes[id] = NetNode(id, x, y, z);;
  if(allNodes[id].exists)
  {
   allNodes[id].x = x;
   allNodes[id].y = y;
   allNodes[id].z = z;
  }
  else
  {
   allNodes[id] = NetNode(id, x, y, z);
  }

}
void NetModel::AddNodeLabel(NodeId_t id, NodeId_t ip_id)
{
   allNodes[id].ip_id = ip_id;
}
void NetModel::OnOffNodeLabel(NodeId_t id)//ip_id)
{
  //NodeId_t id = FindNodeByLabel(ip_id);
  NetNode& n = GetNode(id);
  if (!n.exists)
    {
      cout << "NetModel::OnOffNodeLabel failed - node don't exist" << endl;
      return;
    }
  else if (n.OnOffNodeLabel ())
    {
       allLabels.push_back(id);
       //cout<< " Add label" << endl;
    }
  else {
       RemoveNodeLabel(id);
       //cout<< " Remove label" << endl;
  }
}

NodeId_t NetModel::FindNodeByLabel(NodeId_t ip_id)
{   
    for (uint id=0; id<allLabels.size(); id++)
    {
      if (allNodes[id].ip_id == ip_id)
        {
		return id;
	}
    }
    cout << "Node not found" << endl;
    return -1;
}

void NetModel::RemoveNodeLabel(NodeId_t nId)
{   
    vector<NodeId_t>::iterator it;
    it=allLabels.begin() ;
    while (it != allLabels.end())
    {
      NodeId_t nd = *it;
      if (nd==nId)
        {
           allLabels.erase (it);
           break;
        }
      else it++;
    }
}

void NetModel::AddNode(NodeId_t id,
                             Meters_t x, Meters_t y, Meters_t z, char color)
{
  //cout << "NN " << 0.0 << " " << x0 << " " << y0 << endl;
  if (topLeft == QPointF(0,0) && bottomRight == QPointF(0,0))
    { // First node, set ul and lr
      topLeft = QPointF(x, y);
      bottomRight = QPointF(x, y);
    }
  else
    {
      if (x < topLeft.x())  topLeft.setX(x);
      if (y < topLeft.y())  topLeft.setY(y);
      if (x > bottomRight.x()) bottomRight.setX(x);
      if (y > bottomRight.y()) bottomRight.setY(y);
    }
  allNodes[id] = NetNode(id, x, y, z, color);;
}
/////

NetNode& NetModel::GetNode(NodeId_t id)
{ // Finds a node by id
  return allNodes[id];
}


void NetModel::AddDuplexLink(NodeId_t n1Id, NodeId_t n2Id)
{
  //http://www.sgi.com/tech/stl/Vector.html
  NetNode& n1 = GetNode(n1Id);
  NetNode& n2 = GetNode(n2Id);
  if (!n1.exists || !n2.exists)
    {
      cout << "NetModel::AddDuplexLink::AddDuplexLink failed" << endl;
      return;
    }
  allLinks.push_back(NetWiredLink(n1Id, n2Id));
  //n1.AddLink(allLinks.back());
  allLinks.push_back(NetWiredLink(n2Id, n1Id));
  //n2.AddLink(allLinks.back());
}
void NetModel::DelAllLinks()
{
 allLinks.clear();
 //cout << "alllinks" << endl;
}

void NetModel::DelDuplexLink(NodeId_t nId)
{   
    //for (LinkVec_t::size_type i = 0; i < allLinks.size(); ++i)//for (uint i=0; i<allLinks.size(); i++)
    //for (uint i=0; i<allLinks.size(); i++)
    vector<NetWiredLink>::iterator it, tmp;
    int i=0;
    //for ( it=allLinks.begin() ; it != allLinks.end(); it++ )
    it=allLinks.begin() ;
    while (it != allLinks.end())
    {
      //NetWiredLink lk = allLinks[i];
      NetWiredLink lk = *it;
      if (lk.n2==nId || lk.n1==nId)
        {
         //allLinks.erase (allLinks.begin()+(i));
         if(it==allLinks.begin()){
           //cout << "before errase it  " << (*it).n1 <<" " << (*it).n2 << endl;
           tmp = it+1;
           allLinks.erase (it);
           //cout << "after errase it+1 " << (*tmp).n1 <<" " << (*tmp).n2 << endl;
           it = tmp;
         }
         else
	 {
           //cout << "before errase it  " << (*it).n1 <<" " << (*it).n2 << endl;
           tmp = it-1;
           allLinks.erase (it);
           //cout << "after errase it-1 " << (*tmp).n1 <<" " << (*tmp).n2 << endl;
           it = tmp;
	 }
         //it=allLinks.begin() ;
        }
	else it++;
      i++;
    }
}

NetWiredLink NetModel::GetLink(NodeId_t n1, NodeId_t n2)
{
  NetNode& n = GetNode(n1);
  return n.FindLink(n2);
}


void NetModel::AddPacket(NodeId_t n1Id, NodeId_t n2Id,
                                  Time_t fbTx, Time_t lbTx,
                                  Time_t fbRx, Time_t lbRx)
{
  allPkts.insert(
          make_pair(lbRx, 
                    NetWiredPacket(n1Id, n2Id, fbTx, lbTx, fbRx, lbRx)));
}

// Time Management
void NetModel::AdvanceTime(Time_t t)
{
  currentTime += t;
  // Remove pkts that are in the past
  while(!allPkts.empty())
    {
      multimap<Time_t, NetWiredPacket>::iterator first = allPkts.begin();
      if (first->first > currentTime) return; // Still in future
      // Remove it
      allPkts.erase(allPkts.begin());
    }
}


  
